/* global.h */

#ifndef GLOBAL_H
#define GLOBAL_H

#define VENTIL_KELLER	0	/* zur Indizierung der Bitvariablen  */
#define VENTIL_WC		1	/* (vollflag, halbflag, zuflag, actor_change_flag, timechangeflag) */
#define VENTIL_KUECHE	2	/* und der Variablen-Felder  */    
#define VENTIL_BAD		3	/* (position, direction, move_to, opentime) */
#define VENTIL_STUDIO	4

#define ZWANGSAUS_RESET 5	/* Rcksetzzeit fr Zwangs-Aus Lftung Bad */
							/* (index fr opentime[],  Bitvar. timechangeflag) */

#define GLOBALHALB		7	/* Status alle Ventile 50% [fr actor_change_flag] */

#define NUM_SENSORS		10	/* Anzahl der angeschlossenen Temperatur- und Feuchte-
							   Sensoren x 2 (Temperatur- und Feuchtemessung zhlen
							   als je ein Sensor, auch wenn physikalisch nur ein 
							   Sensor vorhanden ist). */
#define DELAY_SENSOR_READS  3	/* Nach dem Lesen alles Sensoren soll
								   DELAY_SENSOR_READS Sekunden gewartet werden */
/* buffer sizes */
#define UART_RX_BUF_SIZE	254		/* Receive IRQ ring-buffer */
#define UART_TX_BUF_SIZE	132		/* Transmit IRQ buffer */
#define UART_TXI_BUF_SIZE	20		/* Immediate transmit IRQ ring-buffer */
#define RX_TEL_BUF_SIZE		5		/* Receive telegram ring-buffer */
#define TX_TEL_BUF_SIZE		5		/* Transmit telegram ring-buffer */
#define DBG_TX_BUF_SIZE		50	   	/* Ringpuffer fr Soft UART Debug Ausgabe */


typedef unsigned char  u08;
typedef   signed char  s08;
typedef unsigned short u16;
typedef   signed short s16;
typedef unsigned long  u32;
typedef   signed long  s32;
typedef u08            bool;

/* Struktur TimeStamp */
typedef struct
{
	u32 sec;
	u16 xusec;
}TimeStampStruct;


/* The structure that holds the EIB telegrams */
typedef struct 
{
	TimeStampStruct timestamp;	/* timestamp for received telegrams */
	u16 data_length;        	/* length of data (below)
					      		   Note: this is _not_ the same as NPCI[0:3]!!! */
	u08 data[64];               /* telegram data */
} telegram_t;

/* Struktur der Daten im EEPROM */
typedef struct
{
	u16 eib_phys_addr;		/* physikalische Adresse EIB */
	u16 eib_group_start;	/* Erste Gruppenadresse EIB */
	u08 ventilpos_100;		/* ventil position 100% auf */
	u08 ventilpos_50;		/* ventil position 50% auf */
							/* ffnungszeiten manuelle Tastenbettigung in Minuten */
	u16 open_kellerv1;		/* Keller voll #1 */
	u16 open_kellerh1;		/* Keller halb #1 */
	u16 open_wc1;			/* WC EG #1 */
	u16 open_kueche1;		/* Kche #1 */
	u16 open_bad1;			/* Bad #1 */
	u16 open_studio1;		/* WC DG #1 */
							/* (Nach)laufzeiten bei berschreitung Feuchte */
	u16 h_open_keller;		/* Keller */
	u16 h_open_wc;			/* WC EG */
	u16 h_open_kueche;		/* Kche */
	u16 h_open_bad;			/* Bad */
	u16 h_open_studio;		/* WC EG */
	u16 res_force_off_bad;	/* Rcksetzzeit fr Zwangs-Aus Lftung Bad */
							/* Feuchte Grenzwerte */
	s08 humi_gw_keller;		/* Feuchte Grenzwert Keller */
	s08 humi_gw_wc;			/* Feuchte Grenzwert WC EG */
	s08 humi_gw_kueche;		/* Feuchte Grenzwert Kche */
	s08 humi_gw_bad;		/* Feuchte Grenzwert Bad */
	s08 humi_gw_studio;		/* Feuchte Grenzwert WC DG */
}TypeEEPROMStruct;

/* external declarations */
extern TypeEEPROMStruct E2InitData;	
extern u32 global_EIB_object_status;
extern u32 send_EIB1_object_status;
extern u32 send_EIB2_object_status;
extern u08 actor_change_flag;	  
extern s08 sensordata[16];		  
extern u16 sensor_change_flag;	  
extern u08 sensor_read_request_flag;	  
extern u08 active_sensor_string;  
extern u32 next_sensor_read;	  

/* uart globals */
extern u08 uart_rxd_buffer[UART_RX_BUF_SIZE];	 
extern u08 *uart_rxd_in_ptr;					 
extern u08 *uart_rxd_out_ptr;					 
extern u08 *uart_rxd_first_ptr;            		 
extern u08 uart_rxd_received;					 
extern u08 uart_txd_buffer[UART_TX_BUF_SIZE];	 
extern u08 *uart_txd_in_ptr;					 
extern u08 *uart_txd_out_ptr;					 
extern u08 *uart_txd_end_ptr;					 
extern u08 uart_txdi_buffer[UART_TXI_BUF_SIZE];	 
extern volatile u08 uart_txdi_buf_cnt;			 
extern u08 *uart_txdi_in_ptr;					 
extern u08 *uart_txdi_out_ptr;					 
extern telegram_t rx_tel_buffer[RX_TEL_BUF_SIZE];
extern u08 rx_tel_buf_cnt;						 
extern telegram_t *rx_tel_in_ptr;				 
extern telegram_t *rx_tel_out_ptr;				 
extern telegram_t tx_tel_buffer[TX_TEL_BUF_SIZE];
extern u08 tx_tel_buf_cnt;						 
extern telegram_t *tx_tel_in_ptr;
extern telegram_t *tx_tel_out_ptr;
extern volatile u16 rflags, tflags;
extern volatile u08 gflags;
extern volatile TimeStampStruct last_rx;
extern u08 polling_slot;
extern u16 polling_addr;
extern u08 polling_resp;
extern s08 ilength;
extern s16 RXstate;
extern u08 RXchecksum;
extern s08 RXrlength;
extern s16 RXcount;
extern telegram_t RXtemp,TXtemp;


/* timer global variables */
extern volatile u32 seconds;
extern volatile u08 irq20cnt;
extern volatile s08 timeout20cnt;
extern volatile u08 seccnt;
extern volatile u16 opentime[6];
extern volatile u08 position[5];
extern volatile s08 direction[5];
extern u08 move_to[5];
extern u08 luefter_akt;
extern u08 globalhalb;
extern volatile u08 halbflag;
extern volatile u08 vollflag;
extern volatile u08 zuflag;
extern volatile u08 timechangeflag;
extern volatile u08 ser_state;
extern volatile u08 ser_state2;
extern u08 ser_data[3];
extern u08 *dataptr;
extern u08 cmd_byte;
extern s08 which_sensor;
extern u08 u_dbg_buffer[DBG_TX_BUF_SIZE];
extern u08 *u_dbg_in_ptr;					
extern u08 *u_dbg_out_ptr;					
extern u08 u_dbg_cnt;					


/* Bit Funktionen */
//#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
//#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#define sbi(sfr, bit) sfr |= _BV(bit)
#define cbi(sfr, bit) sfr &=~(_BV(bit))

static __inline__ void set_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);

	*p |= mask;
}


static __inline__ void clear_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);

	*p &= ~mask;
}

static __inline__ void change_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);

	*p ^= mask;
}

static __inline__ unsigned char test_and_set_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);
	unsigned char old = *p;

	*p = old | mask;
	return (old & mask) != 0;
}

static __inline__ unsigned char test_and_clear_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);
	unsigned char old = *p;

	*p = old & ~mask;
	return (old & mask) != 0;
}

static __inline__ unsigned char test_and_change_bit(unsigned char nr, volatile void *addr)
{
	unsigned char mask = 1 << (nr & 0x7);
	unsigned char *p = ((unsigned char *)addr) + (nr >> 3);
	unsigned char old = *p;

	*p = old ^ mask;
	return (old & mask) != 0;
}

static __inline__ unsigned char test_bit(unsigned char nr, volatile void *addr)
{
	__const__ unsigned char *p = (__const__ unsigned char *) addr;

	return ((p[nr >> 3] >> (nr & 0x7)) & 1) != 0;
}

#endif
